/* SPDX-License-Identifier: GPL-2.0 */

#ifndef __ASM_GENERIC_SPINLOCK_H
#define __ASM_GENERIC_SPINLOCK_H

#include <asm-generic/ticket_spinlock.h>
#include <asm/qrwlock.h>

#endif /* __ASM_GENERIC_SPINLOCK_H */
