/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2018, The Linux Foundation. All rights reserved.
 */

#ifndef _DT_BINDINGS_CLK_Q6SSTOP_QCS404_H
#define _DT_BINDINGS_CLK_Q6SSTOP_QCS404_H

#define LCC_AHBFABRIC_CBC_CLK			0
#define LCC_Q6SS_AHBS_CBC_CLK			1
#define LCC_Q6SS_TCM_SLAVE_CBC_CLK		2
#define LCC_Q6SS_AHBM_CBC_CLK			3
#define LCC_Q6SS_AXIM_CBC_CLK			4
#define LCC_Q6SS_BCR_SLEEP_CLK			5
#define TCSR_Q6SS_LCC_CBCR_CLK			6

#define Q6SSTOP_BCR_RESET			1
#endif
