/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
#ifndef _DT_BINDINGS_LEDS_LP55XX_H
#define _DT_BINDINGS_LEDS_LP55XX_H

#define LP55XX_CP_OFF		0
#define LP55XX_CP_BYPASS	1
#define LP55XX_CP_BOOST		2
#define LP55XX_CP_AUTO		3

#endif /* _DT_BINDINGS_LEDS_LP55XX_H */
