/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_POWER_SUN60I_A733_PCK600_H_
#define _DT_BINDINGS_POWER_SUN60I_A733_PCK600_H_

#define PD_VI			0
#define PD_DE_SYS		1
#define PD_VE_DEC		2
#define PD_VE_ENC		3
#define PD_NPU			4
#define PD_GPU_TOP		5
#define PD_GPU_CORE		6
#define PD_PCIE			7
#define PD_USB2			8
#define PD_VO			9
#define PD_VO1			10

#endif /* _DT_BINDINGS_POWER_SUN60I_A733_PCK600_H_ */
