/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef __DT_BINDINGS_CLOCK_BCM3368_H
#define __DT_BINDINGS_CLOCK_BCM3368_H

#define BCM3368_CLK_MAC 	3
#define BCM3368_CLK_TC         	5
#define BCM3368_CLK_US_TOP	6
#define BCM3368_CLK_DS_TOP	7
#define BCM3368_CLK_ACM 	8
#define BCM3368_CLK_SPI	        9
#define BCM3368_CLK_USBS	10
#define BCM3368_CLK_BMU         11
#define BCM3368_CLK_PCM         12
#define BCM3368_CLK_NTP         13
#define BCM3368_CLK_ACP_B       14
#define BCM3368_CLK_ACP_A       15
#define BCM3368_CLK_EMUSB       17
#define BCM3368_CLK_ENET0       18
#define BCM3368_CLK_ENET1       19
#define BCM3368_CLK_USBSU       20
#define BCM3368_CLK_EPHY        21

#endif /* __DT_BINDINGS_CLOCK_BCM3368_H */
