/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _MEDIA_JPEG_H_
#define _MEDIA_JPEG_H_

/* JPEG markers */
#define JPEG_MARKER_TEM		0x01
#define JPEG_MARKER_SOF0	0xc0
#define JPEG_MARKER_DHT		0xc4
#define JPEG_MARKER_RST		0xd0
#define JPEG_MARKER_SOI		0xd8
#define JPEG_MARKER_EOI		0xd9
#define JPEG_MARKER_SOS		0xda
#define JPEG_MARKER_DQT		0xdb
#define JPEG_MARKER_DRI		0xdd
#define JPEG_MARKER_DHP		0xde
#define JPEG_MARKER_APP0	0xe0
#define JPEG_MARKER_COM		0xfe

#endif /* _MEDIA_JPEG_H_ */
